function [fb,fh,fw,fi,fo] = simsoftdecode(n,ebn0db)
% [fb,fh,fw,fo] = simsoftdecode(n,ebn0db)
% Simulation von harter und weicher Entscheidung am 
% Beispiel des (7,4)-Hammingcodes:
% n      :  Anzahl der Informationsbits (sollte durch 4 teilbar sein)
% ebn0db :  Standardabweichung des Rauschens
% 
% Ausgegeben wird:
% fb  :  Anzahl der Bitfehler des gesamten codierten Datenstroms
% fh  :  Anzahl der Bitfehler der hart decodierten Information
% fw  :  Anzahl der Bitfehler der weich decodierten Information
% fi  :  Anzahl der Bitfehler der Informationsbits in den Codewrtern
% fo  :  Anzahl der Bitfehler ohne Codierung bei demselben Eb/No

% Autor: K. von der Heide, e-mail: v.d.heide@on-line.de

% Defaults
% ========
if nargin<2  ebn0db = 6; end
if nargin<1  n = 1000;   end

% Konstanten
% ==========
a = 1/sqrt(2*4/7*(10^(ebn0db/10))); % Standardabweichung des Rauschens
codeworte = 2*hamming7_4 - 1;       % Codewrter mit Symbolen {-1,+1}

% Simulation
% ==========
fb = 0;
fh = 0;
fw = 0;
fi = 0;
fo = 0;
for k=1:n/4
   index = ceil(16*rand);           % Auswahl des Codeworts
   info  = 2*int2gf(index-1,4,2)-1; % Informationsbits {-1,+1}
   signal= codeworte(index,:);      % Signal
   ra = a*randn(1,7);               % Rauschen
   se = signal + ra;                % Empfangssignal
   sb = 2*(se>0) - 1;               % harte Bitentscheidung {-1,+1}                
   fb = fb + sum(signal~=sb);       % Anzahl aller Bitfehler (hart)
   hd = codeworte(softdecode(codeworte,sb),:); % harte Decodierung
   sd = codeworte(softdecode(codeworte,se),:); % weiche Decodierung
   fh = fh + sum(info~=hd(1:4));    % Bitfehler harter Decodierung
   fw = fw + sum(info~=sd(1:4));    % Bitfehler weicher Decodierung
   fi = fi + sum(info~=sb(1:4));    % Informationsbits ohne Korrektur
   ra = ra(1:4)*sqrt(4/7);          % Rauschen bei Rate = 1
   se = info + ra;                  % Empfangssignal ohne Codierung                 
   fo = fo + sum(info~=2*(se>0)-1); % Anzahl der Bitfehler ohne Cod.
end


