function [y,Fs,Format]=wavreadx(wavefile)
%WAVREADX  Load Microsoft Windows 3.1 .WAV format sound files.
%   [y]=WAVREADX(wavefile) loads a .WAV format file specified by "wavefile", 
%       returning the sampled data in variable "y". The .WAV extension 
%       in the filename is optional.
%
%   [y,Fs]=WAVREADX(wavefile) loads a .WAV format file specified by  
%       "wavefile", returning the sampled data in variable "y" and the 
%       sample rate in variable "Fs".
%   
%   [y,Fs,Format]=WAVREADX(wavefile) loads a .WAV format file specified by 
%       "wavefile",returning the sampled data in variable "y", the sample 
%       rate in variable "Fs", and the .WAV file format information in 
%       variable "Format". The format information is returned as a 6 element
%       vector with the following order:
%
%               Format(1)       Data format (always PCM) 
%               Format(2)       Number of channels
%               Format(3)       Sample Rate (Fs)
%               Format(4)       Average bytes per second (sampled)
%               Format(5)       Block alignment of data
%               Format(6)       Bits per sample
%
%   Note: WAVREADX currently supports only 8-bit and 16-bit linear data.
%
%   See also WAVWRITE.

%       Copyright (c) 1984-94 by The MathWorks, Inc.
%
%       Modified Jan 1994 by Mike Brookes, EE Dept, Imperial College London.
%       Modifications Copyright (c) 1994 by Mike Brookes
%       (a)     Correct calculation of header chunk lengths
%       (b)     Extension to 16-bit PCM
%       (c)     Data with less than 9 bits has 128 subtracted to give it a mean of zero
%               (data with 9 bits or more is signed in any case)
%       (d)     Supports multiple input channels
%
%       RIFF format definition from Microsoft Multimedia Programmers Reference Guide(1992), Chapt 8
%
%       Minor modification Feb 2000 by Klaus von der Heide in line 47 
%       to suppress an unnecessary warning

if nargin~=1
        error('WAVREAD takes one argument, which is the name of the .WAV file');
end

if isempty(findstr(wavefile,'.'))
        wavefile=[wavefile,'.wav'];
end

fid=fopen(wavefile,'rb','l');
if fid == -1 
        error('Can''t open .WAV file for input!');
else
        % read riff chunk
        header=fread(fid,4,'char');
        header=fread(fid,1,'ulong');
        header=fread(fid,4,'uchar');
  if header' == 'WAVE'
        fmt=0;
        data=0;
    while ~data
        % read a sub-chunk
        header=fread(fid,4,'char');
        len=fread(fid,1,'ulong');
      if header' == 'fmt '
        
        Format(1)=fread(fid,1,'ushort');         % PCM format 
      if Format(1) ~= 1
        fclose(fid);
        error('Not a PCM encoded file!');
      end
        nchan=fread(fid,1,'ushort');             %number of channels
        Format(2) = nchan;
        Fs=fread(fid,1,'ulong');                 % samples per second
        Format(3)=Fs;
        Format(4)=fread(fid,1,'ulong');          % average bytes per second
        Format(5)=fread(fid,1,'ushort');         % block alignment
        Format(6)=fread(fid,1,'ushort');         % bits per sample
        fseek(fid,len-16,0);
        fmt=1;
      elseif header' == 'data'
        if ~fmt
          fclose(fid);
          error('No fmt chunk in RIFF file!');
        end
        datasize=fix((Format(6)+7)/8);
        ksamples = fix(len/(datasize*nchan));
        nsamples = nchan * ksamples;
      if datasize == 1
        y=fread(fid,nsamples,'uchar');
        y=y-128;
      elseif datasize == 2
        y=fread(fid,nsamples,'short');
      else
        fclose(fid);
        error('must be 1 or two byte data!');
      end
        data=1;
        if (nchan > 1)
          y = reshape(y,nchan,ksamples).';
        end
      else
        fseek(fid,len,0);
      end
    end         % end while 
        fclose(fid);
        if ~data
            error('No data chunk in RIFF file!');
        end
  else
        fclose(fid);
        error('First chunck must be WAVE!');
  end                                            % end if header' == 
'WAVE'
end     
